#include <memory> // auto_ptr
#include <sstream>
#include <fstream>
#include <stdexcept>
#include <stdlib.h> // getenv()
#include "url_http.hpp"
#include "pstreams.hpp"
#include <s11n.net/s11n/s11n_debuggering_macros.hpp>
#include <s11n.net/s11n/plugin/path_finder.hpp>

#include <s11n.net/s11n/classload.hpp>
#include <s11n.net/s11n/export.hpp> // unfortunate kludge

namespace s11n { namespace io { namespace pstreams {

	using ::s11n::io::url_parser;

    struct url_http_config
    {
	enum modes { ModeUnknown = 0, ModeWget, ModeCurl };
	static int mode;
	static std::string cmd;
    };

    int url_http_config::mode = url_http_config::ModeUnknown;
    std::string url_http_config::cmd;

    static url_http_config url_http_conf;

	http_stream_factory::http_stream_factory() : ::s11n::io::url_stream_factory()
	{
	}


	http_stream_factory::~http_stream_factory()
	{
	}

	std::ostream *
	http_stream_factory::do_get_ostream( url_parser const & url ) const
	{
		throw ::std::runtime_error( "http_stream_factory cannot write to URLs." );
		return 0;
	}

	std::istream *
	http_stream_factory::do_get_istream( url_parser const & url ) const
	{
		using namespace ::s11n::debug;
		//S11N_TRACE(TRACE_ALWAYS) << "url: "<< url.url() <<'\n';
		if( ! url.good() ) return 0;
		url_http_config & uconf =  url_http_conf;
		std::string cmd = uconf.cmd;
		std::string foo( url.user() );
		if( ! foo.empty() )
		{
			cmd += " --user="+foo;
		}
		foo = url.password();
		if( ! foo.empty() )
		{
		    if( uconf.mode == url_http_config::ModeWget )
		    {
			cmd += " --password="+foo;
		    }
		    else
		    { // curl
			cmd += ":"+foo;
		    }
		}
		char quote = '\'';
		if( std::string::npos != foo.find('\'') )
		{
			quote = '"';
		}
		cmd += " ";
		cmd += quote;
		cmd += url.url();
		cmd += quote;
		//S11N_TRACE(TRACE_ALWAYS) << "http fetching: "<< cmd<<'\n';
		std::istream * is = new redi::ipstream( cmd );
		if( ! is->good() )
		{
		        S11N_TRACE(TRACE_ERROR) << "creation of http stream failed: "<< cmd<<'\n';
			delete is;
			is = 0;
		}
		return is;
	}

	void pstreams_http_static_init()
	{
		/******
		       Register with the classloader and set up some classloader aliases...
		       only if we find wget or curl apps.
		******/

	    url_http_config & uconf =  url_http_conf;
	    uconf.mode = url_http_config::ModeUnknown;

	    using namespace ::s11n::debug;

		::s11n::plugin::path_finder p;
		char * path = ::getenv("PATH");
		p.path( path ? path : "/bin:/usr/bin:/usr/local/bin" );
#if defined(WIN32)
		p.extensions( ".exe" );
#endif
		std::string file = p.find( "wget" );
		if( ! file.empty() )
		{
		    uconf.mode = url_http_config::ModeWget;
		    uconf.cmd = file + " -q -O-";
		}
		else
		{
		    file = p.find( "curl" );
		    if( ! file.empty() )
		    {
			uconf.mode = url_http_config::ModeCurl;
			uconf.cmd = file + " -o -";
		    }
		}
		if( file.empty() )
		{
		    S11N_TRACE(TRACE_ERROR) << "Neither wget nor curl found in $PATH. http URL support disabled.\n";
		    return;
		}

		//S11N_TRACE(TRACE_ALWAYS) << "Using http fetch command: "<< uconf.cmd<<'\n';
		s11n::io::url_stream_factory::register_factory_for_scheme<http_stream_factory>("http");
		s11n::io::url_stream_factory::register_factory_for_scheme<http_stream_factory>("ftp");
	}

	bool pstreams_http_static_init_placeholder = (pstreams_http_static_init(),true);

}}} // namespaces
