#ifndef s11n_addon_PSTREAMS_SSH_HPP_INCLUDED
#define s11n_addon_PSTREAMS_SSH_HPP_INCLUDED 1

#include <string>
#include <s11n.net/s11n/io/url.hpp>
#include <s11n.net/s11n/export.hpp>
#include "license_lgpl.hpp"

namespace s11n { namespace io { namespace pstreams {

	/**
	   This factory creates streams for URLs in the following format:

	   [ssh|scp]://[user@]host:[port:]/path/to/file

	   It works for input and output, and requires that the 'ssh'
	   tool is in your $PATH. For best results, the account you
	   are accessing should have a copy of the requesting user's
	   public SSH key (this prevents the user from being prompted
	   for a password by ssh).
	*/
	class S11N_EXPORT_API ssh_stream_factory : public ::s11n::io::url_stream_factory
	{
	public:
		ssh_stream_factory();
		virtual ~ssh_stream_factory();
	protected:
		/**
		   Returns a stream which writes to url over ssh.

		   The caller owns the returned pointer, which may be 0.
		 */
		virtual std::ostream * do_get_ostream( ::s11n::io::url_parser const & url ) const;

		/**
		   Returns a stream which reads from url over ssh.

		   The caller owns the returned pointer, which may be 0.
		 */
		virtual std::istream * do_get_istream( ::s11n::io::url_parser const & url ) const;
	};

} } } // namespaces



#endif // s11n_addon_PSTREAMS_SSH_HPP_INCLUDED
